<?php
/*
Plugin Name: WPBuzzer
Plugin URI: http://hameedullah.com/wordpress/wpbuzzer
Description: Adds a button to Buzz your blog posts
Version: 0.5
Author: Hameedullah Khan
Author URI: http://hameedullah.com
*/

/*  Copyright 2010  Hameedullah Khan  (email : h@hameedullah.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// display button on all posts and pages
function wpbuzzer_get_button($content) {
    $location = get_option('wpbuzzer-location');
    $showposts = get_option('wpbuzzer-showposts');
    $showpages = get_option('wpbuzzer-showpages');
    $showhome = get_option('wpbuzzer-showhome');
    $showfeed = get_option('wpbuzzer-showfeed');

    // return the content without changing if we can't display the button
    if (is_home() && !$showhome) {
       return $content;
    }
    if (is_single() && !$showposts) {
       return $content;
    }
    if (is_page() && !$showpages) {
       return $content;
    }
    if (is_feed() && !$showfeed) {
       return $content;
    }

    $button_code = wpbuzzer_create_button();
    if ($location == 'before') {
        $content = $button_code. $content;
    } else {
        $content = $content . $button_code;
    }
    return $content;
}

function wpbuzzer_create_button() {
    global $post;

    $post_id = $post->ID;

    // get the post url
    $url = '';
    if (get_post_status($post_id) == 'publish') {
        $permalink = $url = get_permalink($post_id);
    } 

    // get the post title
    $title = get_the_title($post_id);

    // get the blog url
    $blogurl = get_bloginfo('url');

    // get the css
    $css = get_option('wpbuzzer-css');

    // get the size
    $size = get_option('wpbuzzer-size');

    // get the cache time 
    $cachetime = get_option('wpbuzzer-bitlycountcachemtime');
    if (!is_numeric($cachetime)) $cachetime = 60;
    else $cachetime = (int) $cachetime;


    // get tracking information
    $usebitlytracking = get_option('wpbuzzer-usebitlytracking');
    $bitlyapiusername = get_option('wpbuzzer-bitlyapiusername');
    $bitlyapikey = get_option('wpbuzzer-bitlyapikey');

    // reader URL and anchor URL 
    $anchor_url = $reader_url = 'http://www.google.com/reader/link?url=' . $url .
                   '&title=' . $title . '&srcURL=' . $blogurl;

    $count = 0;
    if ($usebitlytracking) {
      /* if tracking is enabled, we have to get shorten URL and stats */
      /* TODO: add caching of count */
      $postcustomfields = get_post_custom($post_id);
      if (array_key_exists("wpbuzzer_bitly_shorturl", $postcustomfields) && $postcustomfields["wpbuzzer_bitly_shorturl"]) {
          $shortenurl = $postcustomfields["wpbuzzer_bitly_shorturl"][0];
      } else {
          $shortenurl = wpbuzzer_create_bitlyurl($reader_url, $bitlyapiusername, $bitlyapikey);
          if (is_string($shortenurl)) {
              add_post_meta($post_id, "wpbuzzer_bitly_shorturl", $shortenurl, true);
          }
      }
      
      // if successfully shorten then set anchor url to bit.ly's shorten url and get count
      if ($shortenurl && is_string($shortenurl)) {
          $anchor_url = $shortenurl;
          // check modified time if cache exists
          $countmtime = array_key_exists("wpbuzzer_share_countmtime", $postcustomfields) ? $postcustomfields["wpbuzzer_share_countmtime"][0] : 0;

          // check cache for share counts 
          if (array_key_exists("wpbuzzer_share_count", $postcustomfields) && ($countmtime + $cachetime) > time()) {
              $count = $postcustomfields["wpbuzzer_share_count"][0];
          } else if (!array_key_exists("wpbuzzer_share_count", $postcustomfields) || ($countmtime + $cachetime) < time()) {
              $count = wpbuzzer_get_bitlycount($shortenurl, $bitlyapiusername, $bitlyapikey);
              if (!update_post_meta($post_id, "wpbuzzer_share_count", $count)) add_post_meta($post_id, "wpbuzzer_share_count", $count, $unique);
              if (!update_post_meta($post_id, "wpbuzzer_share_countmtime", time())) add_post_meta($post_id, "wpbuzzer_share_countmtime", time(), $unique);
          }
      }
    }

    /* we will not use external CSS, to avoid extra calls to the server */
    if ($size == "big") {
        $style = 'style="height: 58px; width:50px; background-image: url(' . WP_PLUGIN_URL . '/WPBuzzer/wpbuzzer-google-buzz-big.png);" class="wpbuzzer_button wpbuzzer_big"';
    } else {
        $style = 'style="height: 20px; width:90px; background-image: url(' . WP_PLUGIN_URL . '/WPBuzzer/wpbuzzer-google-buzz-small.png);" class="wpbuzzer_button wpbuzzer_small"';
    }
    $anchor_tag = '<a href="' . $anchor_url . '" title="Share with Google Buzz" ';

    if (get_option('wpbuzzer-targetwindow') == "popup") {
        $anchor_tag = $anchor_tag . 'onclick="return wpbuzzer_popup(\'' . $anchor_url . '\')" ' . $style. '>';
    } else {
        $anchor_tag = $anchor_tag . 'target="_blank" ' . $style . '>';
    }

    if ($usebitlytracking) {
        $code = '<div class="wpbuzzer_button" style="' . $css . '">' . $anchor_tag . $count . '</a></div>';
    } else {
        $code = '<div class="wpbuzzer_button" style="' . $css . '">' . $anchor_tag . '</a></div>';
    }
    return $code;
}

function wpbuzzer() {
    echo wpbuzzer_create_button();
}

function wpbuzzer_create_bitlyurl($url,$login,$appkey)
{
	// Bit.ly api call
	$bitly = 'http://api.bit.ly/shorten?version=2.0.1&longUrl='.urlencode($url).'&login='.$login.'&apiKey='.$appkey.'&format=json';
	
	// Get the short URL from bitly
	$response = file_get_contents($bitly);
	
        // parse json and return the URL
	$json = @json_decode($response,true);
        if (array_key_exists('results', $json)) {
	    return $json['results'][$url]['shortUrl'];
        } else {
            return false;
        }
	
}

function wpbuzzer_get_bitlycount($shorturl,$login,$appkey)
{
	// Bit.ly api call
	$bitly = 'http://api.bit.ly/stats?version=2.0.1&shortUrl='.urlencode($shorturl).'&login='.$login.'&apiKey='.$appkey.'&format=json';
	
	// Get the stats 
	$response = file_get_contents($bitly);
	
        // parse json and return the URL
	$json = @json_decode($response,true);
        if (array_key_exists('results', $json)) {
	    return $json['results']['clicks'];
        } else {
            return false;
        }
	
}




function wpbuzzer_settings_page() {
?>
    <div class="wrap">
    <h2>WPBuzzer Settings</h2>
    <form method="post" action="options.php">
    <?php
        if (function_exists('settings_field')) {
            settings_field("wpbuzzer-options");
        } else { 
            wp_nonce_field('update-options'); 
        }
    ?>
    <?php
     echo get_option('wpbuzzer-location');
    ?>
    <table class="form-table">
    <tr valign="top">
      <th scope="row">Show On</th>
      <td><input type="checkbox" name="wpbuzzer-showposts" value="1" <?php if (get_option('wpbuzzer-showposts') == '1') { ?>checked="checked"<?php } ?>/>Posts
          <input type="checkbox" name="wpbuzzer-showpages" value="1" <?php if (get_option('wpbuzzer-showpages') == '1') { ?>checked="checked"<?php } ?>/>Pages
          <input type="checkbox" name="wpbuzzer-showhome" value="1" <?php if (get_option('wpbuzzer-showhome') == '1') { ?>checked="checked"<?php } ?>/>Home
          <input type="checkbox" name="wpbuzzer-showfeed" value="1" <?php if (get_option('wpbuzzer-showfeed') == '1') { ?>checked="checked"<?php } ?>/>Feed
      </td>
    <tr valign="top">
      <th scope="row">Location</th>
      <td><input type="radio" name="wpbuzzer-location" value="before" <?php if (get_option('wpbuzzer-location') == 'before') { ?>checked="checked"<?php } ?>/>Before
          <input type="radio" name="wpbuzzer-location" value="after" <?php if (get_option('wpbuzzer-location') == 'after') { ?>checked="checked"<?php } ?>/>After</td>
    </tr>
    <tr valign="top">
      <th scope="row">Target</th>
      <td><input type="radio" name="wpbuzzer-targetwindow" value="popup" <?php if (get_option('wpbuzzer-targetwindow') == 'popup') { ?>checked="checked"<?php } ?>/>Popup
          <input type="radio" name="wpbuzzer-targetwindow" value="newwindow" <?php if (get_option('wpbuzzer-targetwindow') == 'newwindow') { ?>checked="checked"<?php } ?>/>New Window</td>
    </tr>

    <tr valign="top">
      <th scope="row">CSS Style</th>
      <td><input type="text" name="wpbuzzer-css" value="<?php echo get_option('wpbuzzer-css'); ?>" /></td>
    </tr>
    <tr valign="top">
      <th scope="row">Size</th>
      <td><select name="wpbuzzer-size">
          <option value="big" <?php if (get_option('wpbuzzer-size') == 'big') { ?>selected="selected"<?php } ?>>Large</option>
          <option value="small" <?php if (get_option('wpbuzzer-size') == 'small') { ?>selected="selected"<?php }?>>Small</option>
          </select>
      </td>
    </tr>
    <tr valign="top">
      <th scope="row">Track Share Counts (<a href="http://mashable.com/2010/02/12/google-buzz-buttons-count/">Trick from Mashable</a>)</th>
      <td>
          <input type="checkbox" name="wpbuzzer-usebitlytracking" value="1" <?php if (get_option('wpbuzzer-usebitlytracking') == '1' && get_option('wpbuzzer-bitlyapiusername') && get_option('wpbuzzer-bitlyapikey')) { ?>checked="checked"<?php } ?>/>Use Bit.ly To track the share count 
      </td>
    </tr>
    <div id="wpbuzzer-bitlytracking">
    <tr valign="top">
      <th scope="row">Bit.ly API Login</th>
      <td>
          <input type="text" name="wpbuzzer-bitlyapiusername" value="<?php echo get_option('wpbuzzer-bitlyapiusername'); ?>" /><?php if (get_option('wpbuzzer-usebitlytracking') && !get_option('wpbuzzer-bitlyapiusername')) { ?><span class="wpbuzzer-error">You have to provide Bit.ly API Login to use Bit.ly Tracking</span><?php } ?>
      </td>
    </tr>
     <tr valign="top">
      <th scope="row">Bit.ly API Key</th>
      <td>
          <input type="text" name="wpbuzzer-bitlyapikey" value="<?php echo get_option('wpbuzzer-bitlyapikey'); ?>" /><?php if (get_option('wpbuzzer-usebitlytracking') && !get_option('wpbuzzer-bitlyapikey')) { ?><span class="wpbuzzer-error">You have to provide Bit.ly API Key to use Bit.ly Tracking</span><?php } ?>

      </td>
   </tr>
 <tr valign="top">
      <th scope="row">Time to cache Buzz share counts</th>
      <td>
          <input type="text" name="wpbuzzer-bitlycountcachemtime" value="<?php if (is_numeric(get_option('wpbuzzer-bitlycountcachemtime')))  { echo get_option('wpbuzzer-bitlycountcachemtime'); } else { echo "60"; } ?>" /> seconds (Default: 60 seconds)

      </td>
   </tr>

   </div>
    </table>
    <input type="hidden" name="action" value="update" />
    <input type="hidden" name="page_options" value="wpbuzzer-location,wpbuzzer-showposts,wpbuzzer-showpages,wpbuzzer-showhome,wpbuzzer-showfeed,wpbuzzer-css,wpbuzzer-size,wpbuzzer-targetwindow,wpbuzzer-usebitlytracking,wpbuzzer-bitlyapiusername,wpbuzzer-bitlyapikey,wpbuzzer-bitlycountcachemtime" />
    <p class="submit">
    <input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
    </p>
    </form>
    </div>


<?
}

function wpbuzzer_mystyles() {
    $wpbuzzer_styleurl = WP_PLUGIN_URL . '/WPBuzzer/css/style.css';
    $wpbuzzer_stylepath = WP_PLUGIN_DIR . '/WPBuzzer/css/style.css';
    if ( file_exists($wpbuzzer_stylepath) ) {
        wp_register_style('wpbuzzer_stylesheet', $wpbuzzer_styleurl);
        wp_enqueue_style( 'wpbuzzer_stylesheet');
    }
}

function wpbuzzer_myscripts() {
    $wpbuzzer_scripturl = WP_PLUGIN_URL . '/WPBuzzer/js/wpbuzzer.js';
    $wpbuzzer_scriptpath = WP_PLUGIN_DIR . '/WPBuzzer/js/wpbuzzer.js';
    if ( file_exists($wpbuzzer_scriptpath) ) {
        wp_register_script('wpbuzzer_script', $wpbuzzer_scripturl);
        wp_enqueue_script('wpbuzzer_script');
    }
}



function wpbuzzer_activate() {
    add_option('wpbuzzer-location', 'before');
    add_option('wpbuzzer-css', 'float: right');
    add_option('wpbuzzer-size', 'big');
    add_option('wpbuzzer-showposts', 1);
    add_option('wpbuzzer-showpages', 1);
    add_option('wpbuzzer-showhome', 1);
    add_option('wpbuzzer-showfeed', 1);
    add_option('wpbuzzer-targetwindow', "popup");
    add_option('wpbuzzer-usebitlytracking', 0);
    add_option('wpbuzzer-bitlyapiusername', "");
    add_option('wpbuzzer-bitlyapikey', "");
    add_option('wpbuzzer-bitlycountcachemtime', 60);
}

function wpbuzzer_register_settings() {
    if (function_exists('register_settings')) {
        register_settings('wpbuzzer-options', 'wpbuzzer-location');
        register_settings('wpbuzzer-options', 'wpbuzzer-css');
        register_settings('wpbuzzer-options', 'wpbuzzer-size');
        register_settings('wpbuzzer-options', 'wpbuzzer-showposts');
        register_settings('wpbuzzer-options', 'wpbuzzer-showpages');
        register_settings('wpbuzzer-options', 'wpbuzzer-showhome');
        register_settings('wpbuzzer-options', 'wpbuzzer-showfeed');
        register_settings('wpbuzzer-options', 'wpbuzzer-targetwindow');
        register_settings('wpbuzzer-options', 'wpbuzzer-usebitlytracking');
        register_settings('wpbuzzer-options', 'wpbuzzer-bitlyapiusername');
        register_settings('wpbuzzer-options', 'wpbuzzer-bitlyapikey');
        register_settings('wpbuzzer-options', 'wpbuzzer-bitlycountcachemtime');
    }
}

/* add the settings page */
function wpbuzzer_menu() {
    if (function_exists('add_options_page')) {
        add_options_page('WPBuzzer', 'WPBuzzer', 'administrator', basename(__FILE__), 'wpbuzzer_settings_page');
    }
}

if ( is_admin() ){ // admin actions
    add_action( 'admin_init', 'wpbuzzer_register_settings' );
    add_action( 'admin_menu', 'wpbuzzer_menu' );
} 

add_action('wp_print_styles', 'wpbuzzer_mystyles');
add_action('wp_print_scripts', 'wpbuzzer_myscripts');
add_filter('the_content', 'wpbuzzer_get_button', 100);

register_activation_hook( __FILE__, 'wpbuzzer_activate');
?>
